@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2008 Jeffrey Stedfast
+ *  Copyright (C) 2000-2009 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -207,7 +207,7 @@ g_mime_message_partial_new (const char *id, int number, int total)
 	GMimeMessagePartial *partial;
 	char *num;
 	
-	partial = g_object_new (GMIME_TYPE_MESSAGE_PARTIAL, NULL);
+	partial = g_object_newv (GMIME_TYPE_MESSAGE_PARTIAL, 0, NULL);
 	
 	content_type = g_mime_content_type_new ("message", "partial");
 	
