@@ -47,6 +47,7 @@ const guint gmime_interface_age = 0;
 const guint gmime_binary_age = 0;
 
 static unsigned int initialized = 0;
+static guint32 enable = 0;
 
 
 /**
@@ -58,7 +59,7 @@ static unsigned int initialized = 0;
  * Checks that the GMime library version meets the requirements of the
  * required version.
  *
- * Returns %TRUE if the requirement is met or %FALSE otherwise.
+ * Returns: %TRUE if the requirement is met or %FALSE otherwise.
  **/
 gboolean
 g_mime_check_version (guint major, guint minor, guint micro)
@@ -96,6 +97,8 @@ g_mime_init (guint32 flags)
 	tzset ();
 #endif
 	
+	enable = flags;
+	
 	g_type_init ();
 	
 	g_mime_charset_map_init ();
@@ -104,8 +107,17 @@ g_mime_init (guint32 flags)
 	
 	gmime_error_quark = g_quark_from_static_string ("gmime");
 	
+	/* register our GObject types with the GType system */
+	g_mime_message_get_type ();
+	g_mime_content_type_get_type ();
+	g_mime_content_disposition_get_type ();
+	
+	internet_address_get_type ();
+	internet_address_list_get_type ();
+	internet_address_group_get_type ();
+	internet_address_mailbox_get_type ();
+	
 	/* register our default mime object types */
-	g_mime_object_type_registry_init ();
 	g_mime_object_register_type ("*", "*", g_mime_part_get_type ());
 	g_mime_object_register_type ("multipart", "*", g_mime_multipart_get_type ());
 	g_mime_object_register_type ("multipart", "encrypted", g_mime_multipart_encrypted_get_type ());
@@ -129,7 +141,13 @@ g_mime_shutdown (void)
 	if (--initialized)
 		return;
 	
-	g_mime_object_type_registry_shutdown ();
 	g_mime_charset_map_shutdown ();
 	g_mime_iconv_shutdown ();
 }
+
+
+gboolean
+_g_mime_enable_rfc2047_workarounds (void)
+{
+	return (enable & GMIME_ENABLE_RFC2047_WORKAROUNDS);
+}
